/// File: main.dart
library jaguar.example.silly;

import 'dart:async';
import 'dart:io';
import 'package:jaguar/jaguar.dart';
import 'package:jaguar/interceptors.dart';
import 'package:postgresql/postgresql.dart';

import 'package:jaguar_postgresql/jaguar_postgresql.dart';

part 'main.g.dart';

@Api(path: '/api')
class MongoDbExampleApi extends _$JaguarMongoDbExampleApi {
  /// NOTE: Example of using MongoDb interceptor in basic route
  @Get(path: '/post')
  // NOTE: This is how MongoDb interceptor is wrapped
  // around a route.
  // Now other interceptors and route handler methods can
  // request injection of opened connection using `@Input(MongoDb)`
  @WrapPostgresDb('postgres://postgres:dart_jaguar@localhost/postgres')
  @WrapEncodeMapToJson()
  Future<Map> mongoTest(
      // NOTE: This is how the opened mongodb connection is injected
      // into routes
      @Input(PostgresDb) Connection db) async {
    await db.execute("delete FROM posts");
    await db.execute("insert into posts values (1, 'holla', 'jon')");
    Row row = (await db.query("select * from posts WHERE _id = 1").toList()).first;
    return row.toMap();
  }
}

Future<Null> main(List<String> args) async {
  MongoDbExampleApi api = new MongoDbExampleApi();

  Configuration configuration = new Configuration(multiThread: false);
  configuration.addApi(api);

  await serve(configuration);
}