library jaguar_postgres.src.interceptor;

import 'dart:async';

import 'package:jaguar/jaguar.dart';
import 'package:postgresql/postgresql.dart' as pg;
import 'package:connection_pool/connection_pool.dart';

import 'pool.dart';

class WrapPostgresDb implements RouteWrapper<PostgresDb> {
  /// ID for the interceptor instance
  final String id;

  /// Allows making parameter from the context
  final Map<Symbol, MakeParam> makeParams;

  final String uri;

  final int poolSize;

  const WrapPostgresDb(this.uri, {this.poolSize: 10, this.id, this.makeParams});

  /// Creates the interceptor
  PostgresDb createInterceptor() => new PostgresDb(uri, poolSize: poolSize);
}

class PostgresDb extends Interceptor {
  final String uri;

  final int poolSize;

  /// The connection pool
  PostgresqlDbPool _pool;

  /// The connection
  ManagedConnection<pg.Connection> _managedConnection;

  /// Returns the mongodb connection
  pg.Connection get conn => _managedConnection?.conn;

  PostgresDb(this.uri, {this.poolSize: 10});

  Future<pg.Connection> pre() async {
    _pool = new PostgresqlDbPool.Named(uri, poolSize: poolSize);
    _managedConnection = await _pool.getConnection();
    return conn;
  }

  /// Closes the connection on route exit
  void post() {
    _releaseConn();
  }

  /// Closes the connection in case an exception occurs in route chain before
  /// [post] is called
  Future<Null> onException() async {
    _releaseConn();
  }

  /// Releases connection
  void _releaseConn() {
    if (_managedConnection != null) {
      _pool.releaseConnection(_managedConnection);
      _managedConnection = null;
    }
  }
}
